@@ -297,14 +297,16 @@ class WorkflowJob(object):
                     break
                 else:  
                     self._iterate_processing()
-                if len([entry for entry in self.data if entry['type']=='output' and not(entry.has_key('value') or entry.has_key('src'))])== 0:
-                    log.log( 12 , "the workflow is ended (finished ) -> break" )
+                if len([j for j in self.sub_jobs if (not(j.has_key('job_status')) or not(j['job_status'].isEnded()))])==0:
+                    # workflow is considered to be finished when all the tasks have completed
+                    log.log( 12 , "all tasks completed." )
+                    self.log("all tasks completed.")
                     break
                 try:
                     pid = os.wait()
                 except OSError , err :
                     if err.errno == 10 :
-                        log.log( 12 , "no more local child process, may be remote jobs exist -> continue" )
+                        log.log( 12 , "no more local child process, maybe remote jobs exist -> continue" )
                         continue
                     elif err.errno == 4 :
                         log.log( 12 , "interrupt by a system call ( SIGALRM ? ), continue  err = %s time = %f "%( err , time.time() ) )
@@ -337,7 +339,7 @@ class WorkflowJob(object):
         """prepare for executions"""
         self.sub_jobs = []
         for t in self.workflow.tasks:
-            self.sub_jobs.append({'task':t,'job':None,'status':None})
+            self.sub_jobs.append({'task':t})
         # build data transfers scheduling self.data
         self.data = []
         for p in self.workflow.parameters:
@@ -477,7 +479,7 @@ class WorkflowJob(object):
             elif link.to_task:
                 log.debug("from task output to task input")
                 source = [entry for entry in self.data if entry['type']=='task_output' and entry['task'].id==link.from_task and entry['parameter_id']==link.from_parameter][0]
-                target = [entry for entry in self.data if (entry['type']=='task_input' and entry['task'].id==link.to_task)][0]
+                target = [entry for entry in self.data if (entry['type']=='task_input' and entry['task'].id==link.to_task and entry['parameter_id']==link.to_parameter)][0]
             else:
                 log.debug("from task output to workflow output")
                 source = [entry for entry in self.data if entry['type']=='task_output' and entry['task'].id==link.from_task and entry['parameter_id']==link.from_parameter][0]
